/* LICENSE BLOCK
Part of mde: a Modular D game-oriented Engine
Copyright © 2007-2008 Diggory Hardy

This program is free software: you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation, either
version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>. */

/** Some basic OpenGL code for setting up a projection and drawing.
*
* Everything here is really intended as makeshift code to enable GUI development. */
module mde.gl.basic;

import derelict.opengl.gl;

import tango.time.Time;     // TimeSpan (type only; unused)

//BEGIN GL & window setup
void glSetup () {
    glDisable(GL_LIGHTING);
    glDisable(GL_DEPTH_TEST);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
    glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
    glEnable(GL_TEXTURE_2D);
    glShadeModel(GL_SMOOTH);
    
    glClearColor (0.0f, 0.0f, 0.0f, 0.0f);
    
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    
    // Used for font rendering:
    glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
    glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
    //NOTE: wrap mode may have an effect, but shouldn't be noticed...
}

void setProjection (int w, int h) {
    glMatrixMode (GL_PROJECTION);
    glLoadIdentity ();
    
    glViewport (0,0,w,h);
    
    // Make the top-left the origin (see gui/GUI notes.txt):
    // Note that this only affects vertex operations − direct rasterisation operations are
    // unaffected!
    glOrtho (0.0,w, h,0.0, -1.0, 1.0);
    
    glMatrixMode(GL_MODELVIEW);
}
//END GL & window setup

//BEGIN Drawing utils
// Simple drawing commands for use by GUI
// (temporary)
void setColor (float r, float g, float b) {
    glColor3f (r, g, b);
}
void drawBox (int x, int y, int w, int h) {
    glDisable(GL_TEXTURE_2D);
    glRecti(x, y+h, x+w, y);
}
//END Drawing utils
